"""Module holding the Misc. interface class."""

from typing import Optional

from .._backends.misc import MiscBackend
from ..constants import VALID_LANGUAGES
from ..models.locations import Location
from ..models.missions import DroptableNPC, PartialMission


class Misc(MiscBackend):
    """Class for the misc backends."""

    async def get_locations(
        self, language: Optional[VALID_LANGUAGES] = None
    ) -> list[Location]:
        """Gets a list of all locations.

        Parameters
        ----------
        language : Optional[VALID_LANGUAGES]
            The language of the locations. Default: ``None``, meaning the default set during client construction.

        Returns
        -------
        list[Location]

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     locations = await pytenno.misc.get_locations()
        >>>     for location in locations:
        >>>         print(location.node_name)
        """
        return await self._get_locations(language)

    async def get_npcs(
        self, language: Optional[VALID_LANGUAGES] = None
    ) -> list[DroptableNPC]:
        """Gets a list of all NPCs.

        Parameters
        ----------
        language : Optional[VALID_LANGUAGES]
            The language of the NPCs. Default: ``None``, meaning the default set during client construction.

        Returns
        -------
        list[NPC]

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     npcs = await pytenno.misc.get_npcs()
        >>>     for npc in npcs:
        >>>         print(npc.name)
        """
        return await self._get_npcs(language)

    async def get_missions(
        self, language: Optional[VALID_LANGUAGES] = None
    ) -> list[PartialMission]:
        """Gets a list of all missions.

        Parameters
        ----------
        language : Optional[VALID_LANGUAGES]
            The language of the missions. Default: ``None``, meaning the default set during client construction.

        Returns
        -------
        list[PartialMission]

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     missions = await pytenno.misc.get_missions()
        >>>     for mission in missions:
        >>>         print(mission.name)
        """
        return await self._get_missions(language)



from typing import Optional

from ..constants import VALID_LANGUAGES
from ..models.droptable import DropTable
from ..models.items import ItemFull, ItemShort
from ..models.orders import OrderFull, OrderRow
from ..utils import format_name, from_data
from .core import BackendAdapter


class ItemsBackend(BackendAdapter):
    async def _get_items(self, language: str):
        url = "/items"
        headers = {"Language": language}
        response = await self._backend._request(url, headers=headers)

        return [from_data(ItemShort, node) for node in response["payload"]["items"]]

    async def _get_item(
        self,
        item_name: str,
        platform: str,
    ):
        url = f"/items/{format_name(item_name)}"
        headers = {"Platform": str(platform)}
        response = await self._backend._request(url, headers=headers)
        if response is None:
            return None
        items = response["payload"]["item"]["items_in_set"]

        return [from_data(ItemFull, node) for node in items]

    async def _get_orders(
        self,
        item_name,
        include_items,
        platform,
    ):
        url = f"/items/{format_name(item_name)}/orders"
        headers = {"Platform": platform}

        if include_items:
            url += "?include=item"

        response = await self._backend._request(url, headers=headers)
        if include_items:
            return (
                [from_data(OrderRow, node) for node in response["payload"]["orders"]],
                [
                    from_data(ItemFull, node)
                    for node in response["include"]["item"]["items_in_set"]
                ],
            )
        return [from_data(OrderRow, node) for node in response["payload"]["orders"]]

    async def _get_droptable(
        self, item_name, include_items: bool, language: Optional[VALID_LANGUAGES]
    ):
        # url = f"/items/{format_name(item_name)}/droptables"
        url = f"/items/{format_name(item_name)}/dropsources"
        if include_items:
            url += "?include=item"
        headers = {"Language": language}
        response = await self._backend._request(url, headers=headers)
        if include_items:
            return (
                DropTable.from_data(response["dropsources"]),
                [
                    ItemFull.from_data(item)
                    for item in response["include"]["item"]["items_in_set"]
                ],
            )
        return from_data(DropTable, response["payload"]["dropsources"])



from asyncio.proactor_events import _ProactorBasePipeTransport
from functools import wraps


def silence(func):
    @wraps(func)
    def wrap(self, *args, **kwargs):
        try:
            return func(self, *args, **kwargs)
        except RuntimeError:
            pass

    return wrap


_ProactorBasePipeTransport.__del__ = silence(_ProactorBasePipeTransport.__del__)



from ..models.liches import LichEphemera, LichQuirk, LichWeapon
from ..utils import from_data
from .core import BackendAdapter


class LichesBackend(BackendAdapter):
    async def _get_weapons(self, language):
        url = "/lich/weapons"
        headers = {"Language": language}
        response = await self._backend._request(url, headers=headers)
        return [from_data(LichWeapon, node) for node in response["payload"]["weapons"]]

    async def _get_ephemeras(self, language):
        url = "/lich/ephemeras"
        headers = {"Language": language}
        response = await self._backend._request(url, headers=headers)
        return [
            from_data(LichEphemera, node) for node in response["payload"]["ephemeras"]
        ]

    async def _get_quirks(self, language):
        url = "/lich/quirks"
        headers = {"Language": language}
        response = await self._backend._request(url, headers=headers)
        return [from_data(LichQuirk, node) for node in response["payload"]["quirks"]]



"""The main module for the PyTenno client."""

from types import TracebackType
from typing import Optional, Type

import aiohttp

from ._backends.core import PyTennoBackend
from .constants import VALID_LANGUAGES
from .interface.auction_entries import AuctionEntries
from .interface.auctions import Auctions
from .interface.auth import Auth
from .interface.items import Items
from .interface.liches import Liches
from .interface.misc import Misc
from .interface.profile import Profile
from .interface.rivens import Rivens
from .models.enums import Platform


class PyTenno:
    """The primary class for interaction with the warframe.market API endpoints.
    This must be used in an asynchronous context manager.

    Parameters
    ----------
    default_language : str
        The default language used when communicating with the API.
        See ``VALID_LANGUAGES`` for valid values.
    default_platform : Platform
        The default platform used when communicating with the API.
    silenced_errors  : list[Exception]
        A list of errors that will be silenced when raised by the API.
        Instead of raising the error, the function will return None.

    Example
    -------
    >>> async with PyTenno() as tenno:
    >>>     current_user = await tenno.Auth.login(username="username", password="password")
    >>>     print(current_user.ingame_name)
    """

    def __init__(
        self,
        default_language: Optional[VALID_LANGUAGES] = "en",
        default_platform: Platform = Platform.pc,
        silenced_errors: list[Exception] = [],
    ) -> None:
        self._language = default_language
        """The default language used when communicating with the API."""
        self._platform = default_platform
        """The default platform used when communicating with the API."""

        self._session: aiohttp.ClientSession
        """The session used to communicate with the API."""
        self._silenced = silenced_errors
        """A list of errors that will be silenced when raised by the API."""

        self.auction_entries: AuctionEntries
        """The AuctionEntries interface."""
        self.auctions: Auctions
        """The Auctions interface."""
        self.auth: Auth
        """The Auth interface."""
        self.items: Items
        """The Items interface."""
        self.liches: Liches
        """The Liches interface."""
        self.misc: Misc
        """The Misc interface."""
        self.profile: Profile
        """The Profile interface."""
        self.rivens: Rivens
        """The Rivens interface."""

    async def __aenter__(self):
        headers = {
            "Authorization": "JWT",
            "Content-Type": "application/json",
            "Accept": "application/json",
            "Accept-Encoding": "gzip, deflate, br",
            "language": self._language,
            "platform": str(self._platform),
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:103.0) Gecko/20100101 Firefox/103.0"
        }
        self._session = aiohttp.ClientSession(headers=headers)
        backend = PyTennoBackend(self._session, self._silenced)

        self.auction_entries = AuctionEntries(backend)
        self.auctions = Auctions(backend)
        self.auth = Auth(backend)
        self.items = Items(backend)
        self.liches = Liches(backend)
        self.misc = Misc(backend)
        self.profile = Profile(backend)
        self.rivens = Rivens(backend)
        return self

    async def __aexit__(
        self,
        exctype: Optional[Type[BaseException]],
        excinst: Optional[BaseException],
        exctb: Optional[TracebackType],
    ) -> bool:
        await self._session.close()
        return False

    async def close(self) -> None:
        """Closes the client's ``aiohttp`` session."""
        await self._session.close()



from typing import Union

import aiohttp

from ..constants import API_ROOT
from ..utils import _raise_error_code


class PyTennoBackend:
    def __init__(
        self, session: aiohttp.ClientSession, silenced: list[Exception]
    ) -> None:
        self._session = session
        self.silenced = silenced

    async def _request(
        self, url: str, **kwargs
    ) -> dict[str, str | int | dict | list] | None:
        url = f"{API_ROOT}{url}"
        mode: Union[aiohttp.ClientSession.get, aiohttp.ClientSession.post] = getattr(
            self._session, kwargs.pop("method", "get")
        )

        kwargs.setdefault("headers", {})

        if kwargs["headers"].get("Language", None) is None:
            kwargs["headers"]["Language"] = self._session.headers["Language"]

        if kwargs["headers"].get("Platform", None) is None:
            kwargs["headers"]["Platform"] = self._session.headers["Platform"]

        response: aiohttp.ClientResponse = await mode(url, **kwargs)
        if response.status != 200:
            return _raise_error_code(response, self.silenced)

        return await response.json()


class BackendAdapter:
    def __init__(self, backend: PyTennoBackend) -> None:
        self._backend = backend



from ..models.rivens import RivenAttribute, RivenItem
from ..utils import from_data
from .core import BackendAdapter


class RivensBackend(BackendAdapter):
    async def _get_riven_items(self, language):
        url = "/riven/items"
        headers = {"Language": language}
        response = await self._backend._request(url, headers=headers)
        return [from_data(RivenItem, node) for node in response["payload"]["items"]]

    async def _get_riven_attributes(self, language):
        url = "/riven/attributes"
        headers = {"Language": language}
        response = await self._backend._request(url, headers=headers)
        return [
            from_data(RivenAttribute, node)
            for node in response["payload"]["attributes"]
        ]



"""Module holding the Rivens interface class."""

from typing import Optional

from .._backends.rivens import RivensBackend
from ..constants import VALID_LANGUAGES
from ..models.rivens import RivenAttribute, RivenItem


class Rivens(RivensBackend):
    """Class for the rivens backend."""

    async def get_riven_items(
        self, language: Optional[VALID_LANGUAGES] = None
    ) -> list[RivenItem]:
        """Gets a list of all riven-equippable items.

        Parameters
        ----------
        language : Optional[VALID_LANGUAGES]
            The language of the riven items. Default: ``None``, meaning the default set during client construction.

        Returns
        -------
        list[RivenItem]

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     riven_items = await pytenno.rivens.get_riven_items()
        >>>     for riven_item in riven_items:
        >>>         print(riven_item.url_name)
        """
        return await self._get_riven_items(language)

    async def get_attributes(
        self, language: Optional[VALID_LANGUAGES] = None
    ) -> list[RivenAttribute]:
        """Gets a list of all riven attributes.

        Parameters
        ----------
        language : Optional[VALID_LANGUAGES]
            The language of the riven attributes. Default: ``None``, meaning the default set during client construction.

        Returns
        -------
        list[RivenAttribute]

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     riven_attributes = await pytenno.rivens.get_attributes()
        >>>     for riven_attribute in riven_attributes:
        >>>         print(riven_attribute.effect)
        """
        return await self._get_riven_attributes(language)



from dataclasses import dataclass

from .missions import DroptableMission, DroptableNPC, DroptableRelic


@dataclass
class DropTable:
    """Represents an item's drop table."""

    missions: list[DroptableMission]
    """The missions where the item can be found."""
    relics: list[DroptableRelic]
    """The relic in which parts for the item can be found."""
    npc: list[DroptableNPC]
    """The NPCs where the item can be found."""

    def from_data(node: dict):
        return DropTable(
            [DroptableMission.from_data(mission) for mission in node["missions"]],
            [DroptableRelic.from_data(relic) for relic in node["relics"]],
            [DroptableNPC.from_data(npc) for npc in node["npc"]],
        )


@dataclass
class Drop:
    """Represents an item's drop."""

    name: str
    """The translated name of the location / item."""
    link: str
    """Link to the internal or extarnal source with information about that location."""


@dataclass
class RelicRates:
    """Reprents a relic's chance for an item at varius rarities"""

    intact: float
    exceptional: float
    flawless: float
    radiant: float


import argparse
import os

from . import __version__

parser = argparse.ArgumentParser()
parser.add_argument("--version", action="version", version=__version__)
# Create an argument to create a file using basic features of PyTenno
parser.add_argument("--example", action="store_true", help="Create an example file")


args = parser.parse_args()
if isinstance(args, argparse.Namespace):
    if args.example:
        location = input(
            "Please enter a directory in which to save the example file:\n  | "
        )
        path = os.path.abspath(location)
        if not os.path.exists(path):
            os.makedirs(path, exist_ok=True)
        path = os.path.join(path, "example.py")
        with open(path, "w") as f:
            example = """
# Import asyncio to use async/await
import asyncio

# Import pytenno to use the PyTenno class
import pytenno

# Import the Platform enum to specify the default platform to scan on
from pytenno.models.enums import Platform

# Set the defaults for the client. These are not always 
# applicable (such as in this example, where everything is given anyways)
# They are also optional.
defualt_language = "en"
default_platform = Platform.pc

async def main():
    async with pytenno.PyTenno(
        default_language=defualt_language,
        default_platform=default_platform,
    ) as tenno:
        # Use the 'items' interface on the client
        # All methods are split up into the following interfaces:
        #   - auction_entries
        #   - auctions
        #   - auth
        #   - items
        #   - liches
        #   - misc
        #   - profile
        #   - rivens

        # This method returns not only the item, but also items
        # that are similar to the given item, ie. item of the same
        # set.
        items = await tenno.items.get_item("mirage prime set")
        for item in items:
            print(item.en.item_name)

asyncio.run(main())
"""
            f.write(example)
            print(f"Example file created at {path}.")
else:
    print(args)



from ..models.auctions import AuctionEntryExpanded
from ..utils import from_data
from .core import BackendAdapter


class AuctionEntriesBackend(BackendAdapter):
    async def _get_by_id(
        self,
        auction_id: str,
    ):
        data = await self._backend._request(f"/auctions/entry/{auction_id}")
        return from_data(AuctionEntryExpanded, data)

    async def _get_bids_by_id(self, auction_id: str):
        data = await self._backend._request(f"/auctions/entry/{auction_id}/bids")
        return from_data(AuctionEntryExpanded, data)



from dataclasses import dataclass

from .enums import ItemRarity, RelicQuality, Rotation, Stage, Subtype


@dataclass
class DroptableRelic:
    """Represents a relic drop in a mission"""

    id: str
    """The ID of the relic."""
    rarity: ItemRarity
    """The rarity of the item found in the relic."""
    rate: dict[RelicQuality, int | float]
    """A mapping of relic quality to the rate of the relic dropping."""


@dataclass
class DroptableNPC:
    """Represents a NPC in a mission"""

    id: str
    """The ID of the NPC."""
    icon: str
    """The icon URL of the NPC."""
    thumb: str
    """The thumbnail URL of the NPC."""
    name: str
    """The name of the NPC."""


@dataclass
class DroptableMission:
    """Represents a mission."""

    mission_id: str
    """The ID of the mission."""
    node_id: str
    """The ID of the node the mission is on."""
    rarity: ItemRarity
    """The rarity of the item found in the mission."""
    rate: int | float
    """The rate of the item found in the mission."""
    item_subtype: Subtype
    """The subtype of the item found in the mission."""
    rotation: Rotation
    """The rotation where the item can be found."""
    stage: Stage
    """The stage of the item found in the mission."""
    relics: list[DroptableRelic]
    """Relics that can be found in the mission."""
    npc: list[DroptableNPC]
    """The NPCs where the item can be found."""


@dataclass
class PartialMission:
    """Represents a partial mission."""

    id: str
    """The ID of the mission."""
    icon: str
    """The icon URL of the mission."""
    thumb: str
    """The thumbnail URL of the mission."""
    name: str
    """The name of the mission."""



from typing import Literal, Union

from ..models.auctions import (AuctionEntry, AuctionEntryExpanded,
                               KubrowAuction, LichAuction, RivenAuction)
from ..models.enums import Element, Platform, Polarity
from ..utils import format_name, from_data
from .core import BackendAdapter


class AuctionsBackend(BackendAdapter):
    async def _create_auction(
        self,
        item: Union[RivenAuction, LichAuction, KubrowAuction],
        note: str,
        starting_price: int,
        buyout_price: int,
        minimal_reputation: int = 0,
        minimal_increment: int = 1,
        private: bool = False,
    ):
        url = "/auctions/create"
        attributes = [
            {
                [
                    {
                        "positive": attribute.positive,
                        "value": attribute.value,
                        "url": attribute.url_name,
                    }
                ]
                for attribute in item.attributes
            }
        ]
        data = {
            "note": note,
            "starting_price": starting_price,
            "buyout_price": buyout_price,
            "minimal_reputation": minimal_reputation,
            "minimal_increment": minimal_increment,
            "private": private,
            "item": {
                "type": item.type.name.lower(),
                "attributes": attributes,
                "name": item.name.lower(),
                "mastery_level": item.mastery_level,
                "re_rolls": item.re_rolls,
                "weapon_url_name": item.weapon_url_name,
                "polarity": item.polarity.name.lower(),
                "mod_rank": item.mod_rank,
            },
        }

        response = await self._backend._request(url, method="post", data=str(data))
        return from_data(AuctionEntry, response["payload"]["auction"])

    async def _find_riven_auctions(
        self,
        *,
        weapon_url_name: str,
        platform: Platform,
        buyout_policy: Literal["with", "direct"] | None = None,
        mastery_rank_min: int | None = None,
        mastery_rank_max: int | None = None,
        re_rolls_min: int | None = None,
        re_rolls_max: int | None = None,
        positive_stats: list[str] | None = None,
        negative_stats: list[str] | None = None,
        polarity: Polarity = Polarity.any,
        mod_rank: Literal["any", "maxed"] | None = None,
        sort_by: Literal[
            "price_desc", "price_asc", "positive_attr_desc", "positive_attr_asc"
        ]
        | None = None,
        operation: Literal["anyOf", "allOf"] | None = None,
    ):
        url = (
            f"/auctions/search?type=riven&"
            + (
                f"weapon_url_name={weapon_url_name}&"
                if weapon_url_name is not None
                else ""
            )
            + (
                f"mastery_rank_min={mastery_rank_min}&"
                if mastery_rank_min is not None
                else ""
            )
            + (
                f"mastery_rank_max={mastery_rank_max}&"
                if mastery_rank_max is not None
                else ""
            )
            + (f"re_rolls_min={re_rolls_min}&" if re_rolls_min is not None else "")
            + (f"re_rolls_max={re_rolls_max}&" if re_rolls_max is not None else "")
            + (
                f"positive_stats={','.join([str(s) for s in positive_stats])}&"
                if positive_stats is not None
                else ""
            )
            + (
                f"negative_stats={','.join([str(s) for s in negative_stats])}&"
                if negative_stats is not None
                else ""
            )
            + f"polarity={polarity}&"
            + (f"mod_rank={mod_rank}&" if mod_rank is not None else "")
            + (f"sort_by={sort_by}&" if sort_by is not None else "")
            + (f"operation={operation}&" if operation is not None else "")
            + (f"buyout_policy={buyout_policy}" if buyout_policy is not None else "")
        ).strip("&")

        headers = {"Platform": platform.name.lower()}
        response = await self._backend._request(url, headers=headers)

        return [
            from_data(AuctionEntryExpanded, node)
            for node in response["payload"]["auctions"]
        ]

    async def _find_lich_auctions(
        self,
        *,
        platform: Platform,
        weapon_url_name: str,
        element: Element,
        ephemera: bool,
        damage_min: int,
        damage_max: int,
        quirk_url_name: str,
        sort_by: Literal[
            "price_desc", "price_asc", "positive_attr_desc", "positive_attr_asc"
        ] = "price_desc",
        buyout_policy: Literal["with", "direct", "all"] = "all",
    ):
        url = (
            f"/auctions/search?type=lich&"
            + f"weapon_url_name={format_name(weapon_url_name)}&"
            + (f"element={element.name.lower()}&" if element is not None else "")
            + (f"ephemera={ephemera}&" if ephemera is not None else "")
            + (f"damage_min={damage_min}&" if damage_min is not None else "")
            + (f"damage_max={damage_max}&" if damage_max is not None else "")
            + (f"quirk={quirk_url_name}&" if quirk_url_name is not None else "")
            + (f"sort_by={sort_by}&" if sort_by is not None else "")
            + (f"buyout_policy={buyout_policy}" if buyout_policy is not None else "")
        ).strip("&")
        headers = {"Platform": platform.name.lower()}
        response = await self._backend._request(url, headers=headers)

        return [
            from_data(AuctionEntryExpanded, node)
            for node in response["payload"]["auctions"]
        ]



from typing import Optional

from pytenno.models.enums import OrderType, Platform, Subtype
from pytenno.models.items import ItemFull, ItemInOrder
from pytenno.models.orders import OrderCommon, OrderCreated, OrderRow
from pytenno.utils import from_data

from .core import BackendAdapter


class ProfileBackend(BackendAdapter):
    async def _create_order(
        self,
        item_id: str,
        order_type: OrderType,
        platinum: int,
        quantity: int,
        visible: bool,
        subtype: Subtype,
        rank: Optional[int] = None,
    ):
        url = "/profile/orders"
        response = await self._backend._request(
            url,
            method="post",
            json={
                "item_id": item_id,
                "order_type": order_type.name,
                "platinum": platinum,
                "quantity": quantity,
                "visible": visible,
                "subtype": subtype.name,
                "rank": rank,
            },
        )
        order = response["payload"]["order"]
        return from_data(OrderCreated, order)



"""Module holding the Auctions interface class."""

from typing import Literal, Optional, Union

from .._backends.auctions import AuctionsBackend
from ..models.auctions import (AuctionEntryExpanded, KubrowAuction,
                               LichAuction, RivenAuction)
from ..models.enums import Element, Platform, Polarity, RivenStat


class Auctions(AuctionsBackend):
    """A class for creating and searching for auctions with certain criteria."""

    async def create_auction(
        self,
        item: Union[RivenAuction, LichAuction, KubrowAuction],
        note: str,
        starting_price: int,
        buyout_price: int,
        minimal_reputation: Optional[int] = 0,
        minimal_increment: Optional[int] = 1,
        private: Optional[bool] = False,
    ) -> list[AuctionEntryExpanded]:
        """Creates a new auction.

        Parameters
        ----------
        item : RivenAuction, LichAuction, KubrowAuction
            The item to auction.
        note : str
            The note to put on the auction.
        starting_price : int
            The starting price of the auction. (In platinum)
        buyout_price : int
            The buyout price of the auction. (In platinum)
        minimal_reputation : int, optional
            The minimmum reputation a user must have to bid on the auction.
        minimal_increment : int, optional
            The minimum amount between bids. (In platinum)
        private : bool, optional
            Whether the auction is private or not. If it is private, you can set it to public in the auction settings in the web interface.

        Returns
        -------
        list[AuctionEntryExpanded]

        Raises
        ------
        ValueError
            If the starting price is higher than the buyout price.
        ValueError
            If the minimal increment is less than 1.

        Example
        -------
        >>> async with PyTenno() as tenno:
        >>>     auction = await tenno.Auctions.create_auction(
        >>>         item=RivenAuction(...),
        >>>         note="...",
        >>>         starting_price=100,
        >>>         buyout_price=200,
        >>>         minimal_reputation=0,
        >>>         minimal_increment=50
        >>>     )
        >>>     print(auction.owner.ingame_name, auction.platinum)
        """
        if starting_price > buyout_price:
            raise ValueError("Starting price cannot be higher than buyout price.")
        if minimal_increment < 1:
            raise ValueError("Minimal increment cannot be less than 1.")

        return await self._create_auction(
            item,
            note,
            starting_price,
            buyout_price,
            minimal_reputation,
            minimal_increment,
            private,
        )

    async def find_riven_auctions(
        self,
        *,
        weapon_url_name: str,
        platform: Platform = None,
        mastery_rank_min: int = None,
        mastery_rank_max: int = None,
        re_rolls_min: int = None,
        re_rolls_max: int = None,
        positive_stats: list[RivenStat] = None,
        negative_stats: list[RivenStat] = None,
        polarity: Polarity = Polarity.any,
        mod_rank: Literal["any", "maxed"] = None,
        sort_by: Optional[
            Literal[
                "price_desc", "price_asc", "positive_attr_desc", "positive_attr_asc"
            ]
        ] = None,
        operation: Optional[Literal["anyOf", "allOf"]] = None,
        buyout_policy: Optional[Literal["with", "direct"]] = None,
    ) -> list[AuctionEntryExpanded]:
        """Finds all riven auctions that match the given criteria.

        Parameters
        ----------
        weapon_url_name : str
            The URL name of the weapon to search for.
        platform : Platform, optional
            The platform to search for riven auctions on. Default: ``None``, meaning the default set when the client was created.
        mastery_rank_min : int, optional
            The minimum mastery rank of the riven. Default: None.
        mastery_rank_max : int, optional
            The maximum mastery rank of the riven. Default: None.
        re_rolls_min : int, optional
            The minimum number of re-rolls of the riven. Default: None.
        re_rolls_max : int, optional
            The maximum number of re-rolls of the riven. Default: None.
        positive_stats : list[RivenStat], optional
            Restricts the riven to have the given positive stats. Maximum amount is 3. Default: None.
        negative_stats : list[RivenStat], optional
            Restricts the riven to have the given negative stats. Maximum amount is 3. Default: None.
        polarity : Polarity, optional
            The polarity of the riven. Default: ``Polarity.any``.

        Returns
        -------
        list[AuctionEntryExpanded]

        Raises
        ------
        ValueError
            If the amount of ``positive_stats`` is greater than 3.
        ValueError
            If the amount of ``negative_stats`` is greater than 3.
        ValueError
            If the ``mastery_rank_min`` is greater than the ``mastery_rank_max``.
        ValueError
            If the ``re_rolls_min`` is greater than the ``re_rolls_max``.
        ValueError
            If the ``mastery_rank_min`` is less than 0.
        ValueError
            If the ``re_rolls_min`` is less than 0.

        Example
        -------
        >>> auctions = await tenno.auctions.find_riven_auctions(
        >>>     weapon_url_name="shedu",
        >>>     mastery_rank_max=9,
        >>>     re_rolls_max=10
        >>> )
        >>> for auction in auctions:
        >>>     print(auction.id)
        >>>     print(auction.item.element.name)
        """
        if positive_stats is not None and len(positive_stats) > 3:
            raise ValueError("The amount of positive stats cannot be greater than 3.")
        if negative_stats is not None and len(negative_stats) > 3:
            raise ValueError("The amount of negative stats cannot be greater than 3.")
        if mastery_rank_min is not None and mastery_rank_min > mastery_rank_max:
            raise ValueError(
                "The mastery rank min cannot be greater than the mastery rank max."
            )
        if re_rolls_min is not None and re_rolls_min > re_rolls_max:
            raise ValueError(
                "The re-rolls min cannot be greater than the re-rolls max."
            )
        if mastery_rank_min is not None and mastery_rank_min < 0:
            raise ValueError("The mastery rank min cannot be less than 0.")
        if re_rolls_min is not None and re_rolls_min < 0:
            raise ValueError("The re-rolls min cannot be less than 0.")

        return await self._find_riven_auctions(
            platform=platform,
            weapon_url_name=weapon_url_name,
            mastery_rank_min=mastery_rank_min,
            mastery_rank_max=mastery_rank_max,
            re_rolls_min=re_rolls_min,
            re_rolls_max=re_rolls_max,
            positive_stats=positive_stats,
            negative_stats=negative_stats,
            polarity=polarity,
            mod_rank=mod_rank,
            sort_by=sort_by,
            operation=operation,
            buyout_policy=buyout_policy,
        )

    async def find_lich_auctions(
        self,
        *,
        weapon_url_name: str,
        platform: Platform = None,
        element: Optional[Element] = None,
        ephemera: Optional[bool] = None,
        damage_min: Optional[int] = None,
        damage_max: Optional[int] = None,
        quirk_url_name: Optional[str] = None,
        sort_by: Optional[
            Literal[
                "price_desc", "price_asc", "positive_attr_desc", "positive_attr_asc"
            ]
        ] = "price_desc",
        buyout_policy: Optional[Literal["with", "direct"]] = None,
    ) -> list[AuctionEntryExpanded]:
        """Finds all lich auctions that match the given criteria.

        Parameters
        ----------
        weapon_url_name : str
            The URL name of the weapon to search for.
        platform : Platform
            The platform to search for lich auctions on. Default: ``None``, meaning the default set when the client was created.
        element : Element
            The element of the lich. Default: None.
        ephemera : bool
            Whether the lich is ephemeral. Default: None.
        damage_min : int
            The minimum damage of the lich. Default: None.
        damage_max : int
            The maximum damage of the lich. Default: None.
        quirk_url_name : str
            The URL name of the quirk of the lich. Default: None.
        sort_by : Literal
            The sort order of the results. Default: `"price_desc"`.
        buyout_policy : Literal
            The buyout policy of the results. Default: None.

        Returns
        -------
        list[AuctionEntryExpanded]

        Raises
        ------
        ValueError
            If the damage min is greater than the damage max.
        ValueError
            If the damage min is less than 0.
        ValueError
            If the damage max is less than 0.

        Example
        -------
        >>> async with PyTenno() as tenno:
        >>>     auctions = await tenno.auctions.find_lich_auctions(
        >>>         weapon_url_name="kuva_bramma",
        >>>         damage_min=20,
        >>>         element=Element.toxin,
        >>>     )
        >>>     for auction in auctions:
        >>>         print(auction.id, auction.owner.ingame_name)
        """
        if damage_min is not None and damage_min > damage_max:
            raise ValueError("The damage min cannot be greater than the damage max.")
        if damage_min is not None and damage_min < 0:
            raise ValueError("The damage min cannot be less than 0.")
        if damage_max is not None and damage_max < 0:
            raise ValueError("The damage max cannot be less than 0.")

        return await self._find_lich_auctions(
            platform=str(platform),
            weapon_url_name=weapon_url_name,
            element=element,
            ephemera=ephemera,
            damage_min=damage_min,
            damage_max=damage_max,
            quirk_url_name=quirk_url_name,
            sort_by=sort_by,
            buyout_policy=buyout_policy,
        )



from enum import Enum


class Base(Enum):
    """
    Base class for all enums.
    :meta private:
    """

    def __int__(self):
        return self.value

    def __str__(self):
        return self.name

    def __repr__(self):
        return self.name

    def __hash__(self) -> int:
        return hash(self.name)


class RelicQuality(Base):
    """Represents the quality of a relic.

    Attributes
    ----------
    intact : int
        The quality of an intact relic.
    exceptional : int
        The quality of an exceptional relic.
    flawless : int
        The quality of a flawless relic.
    radiant : int
        The quality of a radiant relic.
    """

    intact = 0
    exceptional = 1
    flawless = 2
    radiant = 3


class FishSize(Base):
    """Represents the size of a fish.

    Attributes
    ----------
    small : int
        The size of a small fish.
    medium : int
        The size of a medium fish.
    large : int
        The size of a large fish.
    """

    small = 0
    medium = 1
    large = 2


class FortunaFishQuality(Base):
    """Represents the quality of a fish in fortuna

    Attributes
    ----------
    basic : int
        The quality of a basic fish.
    adorned : int
        The quality of an adorned fish.
    magnificent : int
        The quality of a magnificent fish.
    """

    basic = 0
    adorned = 1
    magnificent = 2


class Subtype(Base):
    """Represents the subtype of an item. Basically a combination of ``RelicQuality``, ``FishSize`` and ``FortunaFishQuality``.

    Attributes
    ----------
    intact : int
        The subtype of an intact relic.
    exceptional : int
        The subtype of an exceptional relic.
    flawless : int
        The subtype of a flawless relic.
    radiant : int
        The subtype of a radiant relic.
    small : int
        The subtype of a small fish.
    medium : int
        The subtype of a medium fish.
    large : int
        The subtype of a large fish.
    basic : int
        The subtype of a basic fortuna fish.
    adorned : int
        The subtype of an adorned fortuna fish.
    magnificent : int
        The subtype of a magnificent fortuna fish.
    """

    intact = 0
    exceptional = 1
    flawless = 2
    radiant = 3
    small = 4
    medium = 5
    large = 5
    basic = 6
    adorned = 7
    magnificent = 8


class ItemRarity(Base):
    """Represents the rarity of an item.

    Attributes
    ----------
    very_common : int
        The rarity of a very common item.
    common : int
        The rarity of a common item.
    uncommon : int
        The rarity of an uncommon item.
    rare : int
        The rarity of a rare item.
    legendary : int
        The rarity of a legendary item.
    peculiar : int
        The rarity of a peculiar item.
    """

    very_common = 0
    common = 1
    uncommon = 2
    rare = 3
    legendary = 4
    peculiar = 5


class OrderType(Base):
    """Represents the type of an order.

    Attributes
    ----------
    buy : int
        The order is buying the item(s).
    sell : int
        The order is selling the item(s).
    """

    buy = 0
    sell = 1


class MeasurementUnit(Base):
    """Represents the measurement unit, for riven attributes.

    Attributes
    ----------
    seconds : int
        The measurement unit is seconds.
    percent : int
        The measurement unit is percent.
    """

    seconds = 0
    percent = 1


class Element(Base):
    """Represents the element of a weapon, ephemera, or lich weapon.

    Attributes
    ----------
    impact : int
        The element is impact.
    heat : int
        The element is heat.
    cold : int
        The element is cold.
    electricity : int
        The element is electricity.
    toxin : int
        The element is toxin.
    magnetic : int
        The element is magnetic.
    radiation : int
        The element is radiation.

    Notes
    -----
    This does **not** include combined elements, aka. ``.heat`` + ``.cold`` -> ``blast``.
    """

    impact = 0
    heat = 1
    cold = 2
    electricity = 3
    toxin = 4
    magnetic = 5
    radiation = 6


class PatreonBadge(Base):
    """Represents the patreon badge of a user.

    Attributes
    ----------
    bronze : int
        The badge is bronze.
    silver : int
        The badge is silver.
    gold : int
        The badge is gold.
    platinum : int
        The badge is platinum.
    """

    bronze = 0
    silver = 1
    gold = 2
    platinum = 3


class Platform(Base):
    """Represents the platform of a user, order, or auction.

    Attributes
    ----------
    ps4 : int
        The platform is on PlayStation 4.
    pc : int
        The platform is on PC.
    xbox : int
        The platform is on Xbox.
    switch : int
        The platform is on Nintendo Switch.
    """

    ps4 = 0
    pc = 1
    xbox = 2
    switch = 3


class UserRole(Base):
    """Represents the role of a user.

    Attributes
    ----------
    anonymous : int
        The user is an anonymous user.
    user : int
        The user is a regular user.
    moderator : int
        The user is a moderator.
    admin : int
        The user is an admin.
    """

    anonymous = 0
    user = 1
    moderator = 2
    admin = 3


class UserStatus(Base):
    """Represents the status of a user.

    Attributes
    ----------
    offline : int
        The user is offline.
    online : int
        The user is online.
    ingame : int
        The user is in Warframe.
    """

    offline = 0
    online = 1
    ingame = 2


class RivenWeaponGroup(Base):
    """Represents group of weapon a riven is on.

    Attributes
    ----------
    primary : int
        The riven is on a primary weapon.
    secondary : int
        The riven is on a secondary weapon.
    melee : int
        The riven is on a melee weapon.
    zaw : int
        The riven is on a zaw.
    sentinel : int
        The riven is on a sentinel weapon.
    archgun : int
        The riven is on an arch gun.
    kitgun : int
        The riven is on a kit gun.
    """

    primary = 0
    secondary = 1
    melee = 2
    zaw = 3
    sentinel = 4
    archgun = 5
    kitgun = 6


class RivenAttributeGroup(Base):
    """Represents the group of an riven attribute. Used for grouping UI elements.

    Attributes
    ----------
    default : int
        The attribute is in the default group.
    melee : int
        The attribute is in the melee group.
    top : int
        The attribute is at the top of the group.
    """

    default = 0
    melee = 1
    top = 2


class RivenWeaponType(Base):
    """Represents the type of the weapon a riven is on.

    Attributes
    ----------
    rifle : int
        The riven in on a rifle-class weapon.
    shotgun : int
        The riven in on a shotgun-class weapon.
    pistol : int
        The riven in on a pistol-class weapon.
    melee : int
        The riven in on a melee-class weapon.
    zaw : int
        The riven in on a zaw-class weapon.
    kitgun : int
        The riven in on a kitgun-class weapon.
    """

    rifle = 0
    shotgun = 1
    pistol = 2
    melee = 3
    zaw = 4
    kitgun = 5


class Polarity(Base):
    """Represents the polarity of a mod.

    Attributes
    ----------
    madurai : int
        The mod is madurai.
    vazarin : int
        The mod is vazarin.
    naramon : int
        The mod is naramon.
    zanurik : int
        The mod is zanurik.
    any : int
        The mod is any polarity. Used in some search scenarios.
    """

    madurai = 0
    vazarin = 1
    naramon = 2
    zanurik = 3
    any = 4
    kitgun = 5


class TranslationLanguage(Base):
    """Represents the language of a translation.

    Attributes
    ----------
    en : int
        The translation is in English.
    ru : int
        The translation is in Russian.
    ko : int
        The translation is in Korean.
    fr : int
        The translation is in French.
    sv : int
        The translation is in Swedish.
    de : int
        The translation is in German.
    zh_hant : int
        The translation is in Chinese (Traditional).
    zh_hans : int
        The translation is in Chinese (Simplified).
    pt : int
        The translation is in Portuguese.
    es : int
        The translation is in Spanish.
    pl : int
        The translation is in Polish.
    """

    en = 0
    ru = 1
    ko = 2
    fr = 3
    sv = 4
    de = 5
    zh_hant = 6
    zh_hans = 7
    pt = 8
    es = 9
    pl = 10


class IconFormat(Base):
    """Represents the format of an icon.

    Attributes
    ----------
    land : int
        The icon is in landscape format.
    port : int
        The icon is in portrait format.
    """

    land = 0
    port = 1


class AnimationFormat(Base):
    """Represents the format of an animation.

    Attributes
    ----------
    land : int
        The animation is in landscape format.
    port : int
        The animation is in portrait format.
    """

    land = 0
    port = 1


class AuctionType(Base):
    """Represents the type of an auction.

    Attributes
    ----------
    riven : int
        The auction is for a riven.
    lich : int
        The auction is for a lich.
    kubrow : int
        The auction is for a kubrow.
    """

    riven = 0
    lich = 1
    kubrow = 2


class AuctionMarking(Base):
    """Represents the marking of an auction.

    Attributes
    ----------
    removing : int
        The auction is marked for removal.
    archiving : int
        The auction is marked for archiving.
    """

    removing = 0
    archiving = 1


class Rotation(Base):
    """Represents the rotation of a stage.

    Attributes
    ----------
    a : int
        The stage is in rotation A.
    b : int
        The stage is in rotation B.
    c : int
        The stage is in rotation C.
    """

    a = 0
    b = 1
    c = 2


class Stage(Base):
    """Represents the stage of a mission.

    Attributes
    ----------
    _1 : int
        The stage is 1.
    _2 : int
        The stage is 2.
    _3 : int
        The stage is 3.
    _4 : int
        The stage is 4.
    final : int
        The stage is the final stage.
    """

    _1 = 0
    _2 = 1
    _3 = 2
    _4 = 3
    final = 4


class Faction(Base):
    """Represents the faction of a mission node.

    Attributes
    ----------
    infested : int
        The node is controlled by the Infested.
    grineer : int
        The node is controlled by the Grineer.
    corpus : int
        The node is controlled by the Corpus.
    corrupted : int
        The node is controlled by the Corrupted.
    """

    infested = 0
    grineer = 1
    corpus = 2
    corrupted = 3


class RivenStat(Base):
    """Represents the stat of a riven attribute.

    Notes
    -----
    The stat names do not always line up with the actual names in Warframe.

    Some names, such as ``.fire_rate_attack_speed``, are modified
    versions of the names in the API, due to limitations of Python.
    Calling the ``.__str__`` method will return the actual name.
    """

    ammo_maximum = 0
    """The stat modifies the maximum ammo of the weapon the riven is on."""
    cold = 1
    """The stat modifies the ``Element.cold`` damage of the weapon the riven is on."""
    critical_chance = 2
    """The stat modifies the critical chance damage of the weapon the riven is on."""
    damage = 3
    """The stat modifies the base damage of the weapon the riven is on."""
    damage_vs_corpus = 4
    """The stat modifies the damage of the weapon the riven is on against the Corpus."""
    damage_vs_grineer = 5
    """The stat modifies the damage of the weapon the riven is on against the Corpus."""
    damage_vs_infested = 6
    """The stat modifies the damage of the weapon the riven is on against the faction."""
    electricity = 7
    """The stat modifies the ``Element.electricity`` damage of the weapon the riven is on."""
    fire_rate_attack_speed = 8
    """The stat modifies the fire rate / attack speed of the weapon the riven is on."""
    heat = 9
    """The stat modifies the ``Element.heat`` damage of the weapon the riven is on."""
    impact = 10
    """The stat modifies the ``Element.impact`` damage of the weapon the riven is on."""
    magazine_capacity = 11
    """The stat modifies the magazine capacity of the weapon the riven is on."""
    multishot = 12
    """The stat modifies the multishot of the weapon the riven is on."""
    projectile_speed = 13
    """The stat modifies the projectile speed of the weapon the riven is on."""
    punch_through = 14
    """The stat modifies the punch through of the weapon the riven is on."""
    puncture = 15
    """The stat modifies the ``Element.puncture`` damage of the weapon the riven is on."""
    reload_speed = 16
    """The stat modifies the reload speed of the weapon the riven is on."""
    slash = 17
    """The stat modifies the ``Element.slash`` damage of the weapon the riven is on."""
    status_chance = 18
    """The stat modifies the status chance of the weapon the riven is on."""
    status_duration = 19
    """The stat modifies the status duration of the weapon the riven is on."""
    toxin = 20
    """The stat modifies the ``Element.toxin`` damage of the weapon the riven is on."""
    weapon_recoil = 21
    """The stat modifies the weapon recoil of the weapon the riven is on."""
    initial_combo = 22
    """The stat modifies the initial combo of the weapon the riven is on."""
    range = 23
    """The stat modifies the range of the weapon the riven is on."""
    chance_to_gain_extra_combo_count = 24
    """The stat modifies the chance for the weapon to gain an extra combo count on a hit."""
    combo_duration = 25
    """The stat modifies the duration of the combo of the weapon the riven is on."""
    critical_chance_on_slide_attack = 26
    """The stat modifies the critical chance on a slide attack of the weapon the riven is on."""
    finisher_damage = 27
    """The stat modifies the finisher damage of the weapon the riven is on."""
    channeling_efficiency = 28
    """The stat modifies the channeling efficiency (heavy attack efficiency) of the weapon the riven is on."""
    channeling_damage = 29
    """The stat modifies the channeling damage (heavy attack damage) of the weapon the riven is on."""

    def __str__(self):
        # Some of these values have non-python-friendly names
        if self.value == 3:
            return "base_damage_/_melee_damage"
        elif self.value == 8:
            return "fire_rate_/_attack_speed"
        return self.name



from ..models.droptable import DroptableNPC
from ..models.locations import Location
from ..models.missions import PartialMission
from ..utils import from_data
from .core import BackendAdapter


class MiscBackend(BackendAdapter):
    async def _get_locations(self, language):
        url = "/misc/locations"
        headers = {"Language": language}
        response = await self._backend._request(url, headers=headers)
        return [from_data(Location, node) for node in response["payload"]["locations"]]

    async def _get_npcs(self, language):
        url = "/misc/npc"
        headers = {"Language": language}
        response = await self._backend._request(url, headers=headers)
        return [from_data(DroptableNPC, node) for node in response["payload"]["npc"]]

    async def _get_missions(self, language):
        url = "/misc/missions"
        headers = {"Language": language}
        response = await self._backend._request(url, headers=headers)
        return [
            from_data(PartialMission, node) for node in response["payload"]["missions"]
        ]



from dataclasses import dataclass, field

from .enums import (IconFormat, MeasurementUnit, RivenAttributeGroup,
                    RivenWeaponGroup, RivenWeaponType)


@dataclass
class RivenItem:
    """Represents a riven item."""

    id: str
    """The ID of the riven item."""
    item_name: str
    """The name of the riven item."""
    url_name: str
    """The URL name of the riven item."""
    group: RivenWeaponGroup
    """The group of the riven item."""
    riven_type: RivenWeaponType
    """The type of the riven item."""
    icon: str
    """The icon URL of the riven item."""
    icon_format: IconFormat
    """The format of the icon URL of the riven item."""
    thumb: str
    """The thumbnail URL of the riven item."""


@dataclass(kw_only=True)
class RivenAttribute:
    """Represents a riven attribute. Most rivens have multiple attributes."""

    id: str
    """The ID of the riven attribute."""
    url_name: str
    """The URL name of the riven attribute."""
    group: RivenAttributeGroup
    """The group of the riven attribute."""
    prefix: str = field(default="")
    """The prefix of the riven attribute. Default is empty string."""
    suffix: str = field(default="")
    """The suffix of the riven attribute. Default is empty string."""
    positive_is_negative: bool
    """Whether the positive attribute is actually negative."""
    exclusive_to: list[RivenWeaponType] | None = None
    """The types of weapons that the attribute is exclusive to. Default is None, meaning it is not exclusive to any weapon."""
    effect: str
    """The name of the effect of the riven attribute. Depends on the requested language."""
    units: MeasurementUnit | None = None
    """The units of the riven attribute. Default is None, meaning it is not measured in any units."""
    negative_only: bool
    """Whether the attribute only appears as a negative."""
    search_only: bool
    """Whether the attribute only appears in search results."""


@dataclass
class PartialRivenAttribute:
    """Represents a partial riven attribute."""

    positive: bool
    """Whether the attribute is positive."""
    value: int
    """The value of the attribute."""
    url_name: str
    """The URL name of the attribute."""



from dataclasses import dataclass

from .enums import AnimationFormat, Element, IconFormat


@dataclass
class LichWeapon:
    """Represents a lich weapon."""

    id: str
    """The ID of the weapon."""
    url_name: str
    """The URL name of the weapon."""
    icon: str
    """The icon URL of the weapon."""
    icon_format: IconFormat
    """The format of the weapon's icon."""
    thumb: str
    """The thumbnail URL of the weapon."""
    item_name: str
    """The name of the weapon."""


@dataclass
class LichEphemera:
    """Represents a lich ephemera."""

    id: str
    """The ID of the ephemera."""
    url_name: str
    """The URL name of the ephemera."""
    icon: str
    """The icon URL of the ephemera."""
    icon_format: IconFormat
    """The format of the ephemera's icon."""
    thumb: str
    """The thumbnail URL of the ephemera."""
    animation: str
    """The animation URL of the ephemera."""
    animation_format: AnimationFormat
    """The format of the ephemera's animation."""
    element: Element
    """The element of the ephemera."""
    item_name: str
    """The name of the ephemera."""


@dataclass
class LichQuirk:
    """Represents a lich quirk."""

    id: str
    """The ID of the quirk."""
    url_name: str
    """The URL name of the quirk."""
    item_name: str
    """The name of the quirk."""
    description: str
    """The description of the quirk."""
    group: str
    """The group of the quirk. Does not belong to any specific enum"""



"""Module holding the Items interface class."""

from typing import Literal, Optional, overload

from .._backends.items import ItemsBackend
from ..constants import VALID_LANGUAGES
from ..models.droptable import DropTable
from ..models.enums import Platform
from ..models.items import ItemFull, ItemShort
from ..models.orders import OrderRow


class Items(ItemsBackend):
    """Class for the items backend."""

    async def get_items(
        self, language: Optional[VALID_LANGUAGES] = None
    ) -> list[ItemShort]:
        """Gets all items.

        Parameters
        ----------
        language : Optional[VALID_LANGUAGES]
            The language of the items. Default: ``None``, meaning the default set during client construction.

        Returns
        -------
        list[ItemShort]

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     items = await pytenno.items.get_items()
        >>>     for item in items:
        >>>         print(item.url_name)
        """
        return await self._get_items(language)

    async def get_item(
        self,
        item_name: str,
        *,
        platform: Optional[Platform] = None,
    ) -> list[ItemFull]:
        """Gets the item with the given name, as well as related items (such as items of the same set).

        The item must be tradeable.

        Parameters
        ----------
        item_name : str
            The name of the item.
        platform : Platform
            The platform of the item. Default: ``None``, meaning the default set when the client was created.

        Returns
        -------
        list[ItemFull]

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     items = await pytenno.items.get_item("mirage prime set")
        >>>     for item in items:
        >>>         print(item.url_name)
        """
        return await self._get_item(item_name, platform)

    @overload
    async def get_orders(
        self,
        item_name: str,
        include_items: Literal[False],
        platform: Optional[Platform] = None,
    ) -> list[OrderRow]:
        ...

    @overload
    async def get_orders(
        self,
        item_name: str,
        include_items: Literal[True],
        platform: Optional[Platform] = None,
    ) -> tuple[list[OrderRow], list[ItemFull]]:
        ...

    async def get_orders(
        self,
        item_name: str,
        include_items: bool,
        platform: Optional[Platform] = None,
    ):
        """Gets the orders of the given item.

        Parameters
        ----------
        item_name : str
            The name of the item.
        include_items : bool
            Whether to include information about the item requested.
        platform : Platform
            The platform of the item. Default: ``None``, meaning the default set when the client was created.

        Returns
        -------
        list[OrderRow] | tuple(list[OrderRow], list[ItemFull])

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     orders, items = await pytenno.items.get_orders("mirage prime set", include_items=True)
        >>>     for order in orders:
        >>>         print(order.user.ingame_name)
        >>>     for item in items:
        >>>         print(item.url_name)
        """
        return await self._get_orders(item_name, include_items, str(platform))

    @overload
    async def get_droptable(
        self,
        item_name: str,
        include_items: Literal[False],
        language: Optional[VALID_LANGUAGES] = None,
    ) -> DropTable:
        ...

    @overload
    async def get_droptable(
        self,
        item_name: str,
        include_items: Literal[True],
        language: Optional[VALID_LANGUAGES] = None,
    ) -> tuple[DropTable, list[ItemFull]]:
        ...

    async def get_droptable(
        self,
        item_name: str,
        include_items: bool,
        language: Optional[VALID_LANGUAGES] = None,
    ):
        """Gets the droptable of the given item.

        Parameters
        ----------
        item_name : str
            The name of the item.
        include_items : bool
            Whether to include information about the item requested.
        language : Optional[VALID_LANGUAGES]
            The language of the droptable. Default: ``None``, meaning the default set during client construction.

        Returns
        -------
        DropTable | tuple(DropTable, list[ItemFull])

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     droptable, items = await pytenno.items.get_droptable("frost prime neuroptics", include_items=True)
        >>>     print(droptable.relics, droptable.missions)
        >>>     for item in items:
        >>>         print(item.url_name)

        """
        return await self._get_droptable(item_name, include_items, language)



"""Module for utility functions used elsewhere in the pytenno package.

    Some of these functions are used to convert data types to and from
    their respective JSON representations.

    There are also a handful of user-friendly functions for common
    tasks, such as converting a string to a valid url-safe string.
"""


import datetime
from enum import Enum
from typing import Any, Callable, Mapping, Type, TypeVar
from urllib.parse import quote

import aiohttp

from .constants import ASSET_ROOT, VALID_TRANSLATIONS_RAW
from .errors import BaseError
from .models.droptable import Drop, RelicRates
from .models.enums import (AnimationFormat, AuctionMarking, AuctionType,
                           Element, Faction, IconFormat, ItemRarity,
                           MeasurementUnit, OrderType, PatreonBadge, Platform,
                           Polarity, RivenAttributeGroup, RivenWeaponGroup,
                           RivenWeaponType, Rotation, Stage, Subtype, UserRole,
                           UserStatus)


def format_name(name: str):
    """Converts a string to a valid url-safe string."""
    return quote(name.lower().replace(" ", "_"))


def is_formatted_name(name: str):
    """Checks if a string is a valid formatted name."""
    return quote(name.replace("_", " ").lower()) == name


def _raise_error_code(response: aiohttp.ClientResponse, silenced: list[Exception]):
    """Raises an error based on the response's status code."""
    code = response.status

    for error in BaseError.__subclasses__():
        if error.code == code:
            if error not in silenced:
                raise error
            return None

    error = BaseError
    error.code = code
    error.msg = "Unknown error occurred"

    raise error


DATETIME_FORMAT: str = "%Y-%m-%dT%H:%M:%S.%f%z"


# Enums that take in a value and return an enum value
_ENUM_MAPPING: Mapping[str, Type[Enum]] = {
    "rarity": ItemRarity,
    "order_type": OrderType,
    "element": Element,
    "patreon_badge": PatreonBadge,
    "platform": Platform,
    "role": UserRole,
    "status": UserStatus,
    "polarity": Polarity,
    "riven_type": RivenWeaponType,
    "icon_format": IconFormat,
    "animation_format": AnimationFormat,
    "rotation": Rotation,
    "type": AuctionType,
    "is_marked_for": AuctionMarking,
    "faction": Faction,
}

# Enums that require special attention
_SPECIAL_ENUM_MAPPING: Mapping[str, Callable[[str], Type[Enum]]] = {
    "subtypes": lambda names: [Subtype[name] for name in names],
    "exclusive_to": lambda excls: [RivenWeaponGroup[exc] for exc in excls],
    "units": lambda unit: MeasurementUnit[unit],
    "group": lambda grp: RivenAttributeGroup[
        grp
    ]  # The API is ambigous on this attribute; three different items have the name but can have different values
    if hasattr(RivenAttributeGroup, grp)
    else RivenWeaponGroup[grp]
    if hasattr(RivenWeaponGroup, grp)
    else grp,
    "stage": lambda stage: Stage[f"_{stage}" if stage.isdigit() else stage],
    "is_marked_for": lambda mark: AuctionMarking[mark] if mark is not None else None,
    "rates": lambda rates: RelicRates(**rates),
}

T = TypeVar("T", bound=type)


def from_data(
    cls_: Type[T], data: dict[str, Any] | list[dict[str, Any]] | None, use_data_method: bool = True
) -> T:
    """Partially converts common data types into their object equivalent, then creates an instance of ``cls_``."""
    if data is None:
        return None
    
    if isinstance(data, list):
        for d in data:
            d = _format_names(d)
    else:
        data = _format_names(data)

    if hasattr(cls_, "from_data") and use_data_method:
        return cls_.from_data(data)
    return cls_(**data)


def _format_names(d: dict):
    nd = {}  # Create new dict to avoid RuntimeErrors
    for key, value in d.items():
        if value is None:
            continue
        if key in ("icon", "sub_icon", "thumb", "avatar", "animation", "background"):
            nd[key] = f"{ASSET_ROOT}/{value}"
        elif key in (
            "creation_date",
            "created",
            "last_updated",
            "last_update",
            "updated",
            "last_seen",
            "marked_operation_at",
        ):
            nd[key] = datetime.datetime.strptime(value, DATETIME_FORMAT)
        elif key == "drop":
            nd[key] = [Drop(**val) for val in value]
        elif key in {"zh-hans", "zh-hant"}:
            nd[{"zh-hans": "zh_hans", "zh-hant": "zh_hant"}[key]] = value
        else:
            try:
                nd[key] = _ENUM_MAPPING[key][value]
            except KeyError:
                try:
                    nd[key] = _SPECIAL_ENUM_MAPPING[key](value)
                except KeyError:
                    nd[key] = value
    return nd




from ..models.users import CurrentUser
from ..utils import from_data
from .core import BackendAdapter


class AuthBackend(BackendAdapter):
    async def _login(
        self,
        email: str,
        password: str,
        device_id: str,
    ):
        url = "/auth/signin"
        data = {
            "auth_type": "header",
            "email": email,
            "password": password,
            "device_id": device_id,
        }
        response = await self._backend._request(url, json=data, method="post")
        return from_data(CurrentUser, response["payload"]["user"])

    async def _register(
        self, email: str, password: str, region: str, device_id: str, recaptcha: str
    ):
        url = "/auth/registration"
        data = {
            "auth_type": "header",
            "email": email,
            "password": password,
            "region": region,
            "device_id": device_id,
            "recaptcha": recaptcha,
        }
        response = await self._backend._request(url, json=data, method="post")
        return from_data(CurrentUser, response)

    async def _restore(self, email: str):
        url = "/auth/restore"
        data = {
            "email": email,
        }
        await self._backend._request(url, json=data, method="post")
        return None



"""Module holding the Profile interface class."""

from typing import Optional

from .._backends.profile import ProfileBackend
from ..models.enums import OrderType, Subtype
from ..models.orders import OrderCreated


class Profile(ProfileBackend):
    """Class for the profile backend."""

    async def create_order(
        self,
        item_id: str,
        order_type: OrderType,
        platinum: int,
        quantity: int,
        visible: bool,
        subtype: Subtype,
        rank: Optional[int] = None,
    ) -> OrderCreated:
        """Creates an order, and returns the order object.

        Parameters
        ----------
        item_id : str
            The ID of the item.
        order_type : OrderType
            The type of order.
        platinum : int
            The amount of platinum per item in the order.
        quantity : int
            How many items to offer to buy / sell.
        visible : bool
            Whether the order is visible to others.
        subtype : Subtype
            The subtype of the item. Must be applicable to the item passed.
        rank : Optional[int]
            The rank of the item. Unnecessary for anything which doesn't have a rank.

        Returns
        -------
        OrderCreated
            The order object.
        """
        return await self._create_order(
            item_id,
            order_type,
            platinum,
            quantity,
            visible,
            subtype,
            rank,
        )



"""Module holding the Auth interface class."""

from typing import Any, Coroutine, Optional

from .._backends.auth import AuthBackend
from ..constants import VALID_LANGUAGES
from ..models.users import CurrentUser


class Auth(AuthBackend):
    """Class for the authentication backend."""

    async def login(
        self,
        *,
        email: str,
        password: str,
        device_id: Optional[str] = None,
    ) -> CurrentUser:
        """Logs in the user with the given credentials.

        Parameters
        ----------
        email : str
            The email of the user.
        password : str
            The password of the user.
        device_id : str, optional
            The device ID of the user. This can be used to recognize the device between sessions. Default: None.

        Returns
        -------
        CurrentUser

        Example
        -------
        >>> async with PyTenno() as tenno:
        >>>     current_user = await tenno.auth.login(
        >>>         email="example@nothing.co"
        >>>         password="password"
        >>>     )
        >>>     print(current_user.ingame_name)
        """
        return await self._login(email, password, device_id)

    async def register(
        self,
        *,
        email: str,
        password: str,
        region: Optional[VALID_LANGUAGES] = None,
        device_id: Optional[str] = None,
        recaptcha: Optional[str] = None,
    ) -> CurrentUser:
        """Registers a new user with the given credentials.

        Parameters
        ----------
        email : str
            The email of the user.
        password : str
            The password of the user.
        region : Optional[VALID_LANGUAGES]
            The region of the user. Default: ``None``, meaning the default set during client construction.
        device_id : str
            The device ID of the user, used to identify devices between sessions. Default: None.
        recaptcha : str
            The Google recaptcha response of the user. Default: None.

        Returns
        -------
        CurrentUser

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     email = "example@nothing.co"
        >>>     password = "password"
        >>>     region = "en"
        >>>     current_user = await pytenno.auth.register(email, password, region)
        >>>     print(current_user.ingame_name)

        """
        return await self._register(email, password, region, device_id, recaptcha)

    async def restore(self, email: str) -> None:
        """ "Sends the user a recovery email.

        Parameters
        ----------
        email: str
            The email of the user.

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     email = "example@nothing.co"
        >>>     await pytenno.auth.recover(email=email)
        """
        return await self._restore(email)



"""Small module for constants used throughout the PyTenno library."""

__all__ = ["VALID_LANGUAGES", "MAIN_ROOT", "API_ROOT", "ASSET_ROOT"]

from typing import Final, Literal

MAIN_ROOT: Final[str] = "https://warframe.market"
API_ROOT: Final[str] = "https://api.warframe.market/v1"
ASSET_ROOT: Final[str] = "https://warframe.market/static/assets"

VALID_LANGUAGES = Literal[
    "en",
    "ru",
    "ko",
    "fr",
    "sv",
    "de",
    "zh_hans",
    "zh_hant",
    "pt",
    "es",
    "pl",
]

VALID_TRANSLATIONS_RAW = {
    "en",
    "ru",
    "ko",
    "fr",
    "sv",
    "de",
    "zh_hans",
    "zh_hant",
    "pt",
    "es",
    "pl",
}



"""Module holding the AuctionEntries interface class."""

from .._backends.auction_entries import AuctionEntriesBackend
from ..models.auctions import AuctionEntryExpanded


class AuctionEntries(AuctionEntriesBackend):
    """The AuctionEntries interface class."""

    def __init__(self, backend: AuctionEntriesBackend) -> None:
        """Initializes the interface class. For internal use."""
        super().__init__(backend)

    async def get_by_id(self, auction_id: str) -> AuctionEntryExpanded:
        """Gets a specific auction entry by ID.

        Parameters
        ----------
        auction_id : str
            The ID of the auction entry to get.

        Returns
        -------
        AuctionEntryExpanded

        Example
        -------
        >>> async with PyTenno() as tenno:
        >>>     auction = await tenno.AuctionEntries.get_by_id("...")
        >>>     print(auction.owner.ingame_name, auction.platinum)
        """
        return await self._get_by_id(auction_id)

    async def get_bids_by_id(self, auction_id: str) -> AuctionEntryExpanded:
        """Gets all bids for a specific auction entry by ID.

        Parameters
        ----------
        auction_id : str
            The ID of the auction entry to get bids for.

        Returns
        -------
        AuctionEntryExpanded

        Example
        -------
        >>> async with PyTenno() as tenno:
        >>>     auction = await tenno.AuctionEntries.get_bids_by_id("...")
        >>>     print(auction.owner.ingame_name, auction.platinum)
        """
        return await self._get_bids_by_id(auction_id)



from dataclasses import dataclass
from datetime import datetime

from pytenno.utils import from_data

from .enums import OrderType, Platform
from .items import ItemInOrder
from .users import UserShort


@dataclass(kw_only=True)
class OrderCommon:
    """Common base class that orders inherit from."""

    id: str
    """The ID of the order."""
    platinum: int
    """The amount of platinum per item in the order."""
    quantity: int
    """How many items the user is selling / buying."""
    order_type: OrderType
    """The type of order."""
    platform: Platform
    """The platform the order is on."""
    region: str
    """The region the order is on."""
    creation_date: datetime
    """The time the order was created."""
    last_update: datetime
    """The time the order was last updated."""
    visible: bool
    """Whether the order is visible to others. In this case, always True."""
    mod_rank: int | None = None
    """The mod rank of the order's item. Default is ``None``, meaning not applicable."""


@dataclass(kw_only=True)
class OrderCreated(OrderCommon):
    """Represents an Order after it has been passed to the API
    and is now created."""

    item: ItemInOrder
    """The item in the order."""

    def from_data(data: dict):
        return OrderCreated(
            item=from_data(ItemInOrder, data.pop("item")),
        )


@dataclass(kw_only=True)
class OrderRow(OrderCommon):
    """Same as OrderCommon, but with a full user model."""

    user: UserShort
    """The user who made the order."""

    def from_data(node: dict):
        return OrderRow(
            user=UserShort(**node.pop("user")),
            **node,
        )

    def __repr__(self) -> str:
        return f"<OrderRow id={self.id} user={self.user.ingame_name}>"


@dataclass(kw_only=True)
class OrderFull(OrderRow):
    """Same as OrderRow, but with a full item model."""

    item: ItemInOrder
    """The item in the order."""



from dataclasses import dataclass
from datetime import datetime

from ..utils import from_data
from .enums import PatreonBadge, Platform, UserRole, UserStatus


@dataclass
class PatreonProfile:
    """Represents a Patreon profile."""

    patreon_founder: bool
    """Whether the user is a Patreon founder."""
    subscription: bool
    """Whether the user is a Patreon subscriber."""
    patreon_badge: PatreonBadge | None = None
    """The Patreon badge of the user, if any."""


@dataclass
class BaseUser:
    """Base class that other user classes inherit from."""

    id: str
    """The ID of the user."""
    ingame_name: str
    """The ingame name of the user."""
    region: str
    """The region the user is on."""
    avatar: str | None = None
    """The URL of the user's avatar, if any."""

    def __repr__(self):
        return f"<User id={self.id} ingame_name={self.ingame_name}>"


@dataclass
class LinkedProfiles:
    """Represents a user's linked profiles."""

    discord_profile: bool
    """Whether the user has a Discord profile linked to their warframe.market profile."""
    patreon_profile: bool
    """Whether the user has a Patreon profile linked to their warframe.market profile."""
    xbox_profile: bool
    """Whether the user has a Xbox profile linked to their warframe.market profile."""
    steam_profile: bool
    """Whether the user has a Steam profile linked to their warframe.market profile."""


@dataclass(kw_only=True)
class CurrentUser(BaseUser):
    """Represents the current user. This is the user that is logged in."""

    anonymous: bool
    """Whether the user is anonymous."""
    verification: bool
    """Whether the user is verified."""
    check_code: str
    """The check / verification code of the user."""
    role: UserRole
    """The role of the user."""
    platform: Platform
    """The platform of the user."""
    banned: bool
    """Whether the user is banned."""
    ban_reason: str | None = None
    """The reason the user is banned. If None, the user is not banned."""
    background: str | None = None
    """The URL of the user's background. If None, the user has no background."""
    has_mail: bool
    """Whether the user has unread mail."""
    reputation: int
    """The reputation of the user."""
    linked_accounts: LinkedProfiles
    """The linked accounts of the user."""
    patreon_profile: PatreonProfile
    """The Patreon profile of the user."""
    written_reviews: int
    """The number of reviews the user has written today."""
    unread_messages: int
    """The number of unread messages the user has."""

    def from_data(node: dict):
        return CurrentUser(
            # file deepcode ignore WrongNumberOfArguments
            patreon_profile=from_data(PatreonProfile, node.pop("patreon_profile")),
            linked_accounts=LinkedProfiles(**node.pop("linked_accounts")),
            **node,
        )


@dataclass(kw_only=True)
class UserShort(BaseUser):
    """Represents a user."""

    status: UserStatus
    """The status of the user."""
    reputation: int
    """The reputation of the user."""
    last_seen: datetime | None
    """The last time the user was seen. If None, the user has not been seen."""



from dataclasses import dataclass
from datetime import datetime
from typing import Union

from ..utils import from_data
from .enums import AuctionMarking, AuctionType, Element, Platform, Polarity
from .rivens import PartialRivenAttribute
from .users import UserShort


@dataclass(kw_only=True)
class AuctionEntry:
    """Represents an auction entry."""

    id: str
    """The ID of the auction entry."""
    minimal_reputation: int
    """The minimal reputation required to bid on the auction."""
    winner: str | None = None
    """The ID of the winner of the auction, if any."""
    private: bool
    """Whether the auction is private or not."""
    visible: bool
    """Whether the auction is visible or not."""
    note_raw: str
    """The raw note of the auction."""
    note: str
    """The note of the auction entry."""
    owner: str
    """The ID of the owner of the auction."""
    starting_price: int
    """The starting price of the auction."""
    buyout_price: int | None = None
    """The buyout price of the auction, if any."""
    minimal_increment: int | None = None
    """The minimal increment for bids, if any."""
    is_direct_sell: bool
    """Whether the auction is a direct sell or not."""
    top_bid: int | None = None
    """The highest bid on the auction, if any."""
    created: datetime
    """The date and time the auction was created."""
    updated: datetime
    """The date and time the auction entry was created."""
    platform: Platform
    """The platform the auction is on."""
    closed: bool
    """Whether the auction is closed or not."""
    is_marked_for: AuctionMarking | None = None
    """Whether the auction is marked for a specific action."""
    marked_operation_for: datetime | None = None
    """The time when the auction was marked for a specific operation."""
    item: Union["RivenAuction", "LichAuction", "KubrowAuction"]
    """The item of the auction."""


@dataclass
class AuctionEntryExpanded(AuctionEntry):
    """Same as `AuctionEntry`, but with a full user model for ``.owner``"""

    owner: UserShort
    """The owner of the auction."""

    def from_data(node: dict):
        # deepcode ignore
        return AuctionEntryExpanded(
            # file deepcode ignore WrongNumberOfArguments
            owner=from_data(UserShort, node.pop("owner")),
            item=from_data(RivenAuction, item)
            if (t := (item := node.pop("item"))["type"]) == "riven"
            else from_data(LichAuction, item)
            if t == "lich"
            else from_data(KubrowAuction, item),
            **node,
        )


@dataclass(kw_only=True)
class LichAuction:
    """Represents a lich auction."""

    type: AuctionType  # lich
    """The type of the auction. In this case, ``lich``."""
    weapon_url_name: str
    """The URL name of the weapon."""
    element: Element
    """The element of the weapon."""
    damage: int
    """The damage of the weapon."""
    having_ephemera: bool
    """Whether the weapon has an ephemera."""
    quirk: str | None = None
    """The quirk of the lich, if any."""
    name: str | None = None
    """The name of the lich. Unused by the API."""


@dataclass
class KubrowAuction:
    """Represents a kubrow auction."""

    type: AuctionType
    """The type of the auction. In this case, ``kubrow``."""
    name: str
    """The name of the kubrow."""


@dataclass
class RivenAuction:
    """Represents a riven auction."""

    type: AuctionType
    """The type of the auction. In this case, ``riven``."""
    attributes: list[PartialRivenAttribute]
    """The attributes of the riven."""
    name: str
    """The name of the riven."""
    mastery_level: int
    """The mastery level of the riven."""
    re_rolls: int
    """The number of times the riven has been rerolled."""
    weapon_url_name: str
    """The URL of the weapon the riven is for."""
    polarity: Polarity
    """The polarity of the riven."""
    mod_rank: int
    """The rank of the riven."""

    def from_data(node: dict):
        return RivenAuction(
            attributes=[
                from_data(PartialRivenAttribute, x) for x in node.pop("attributes")
            ],
            **node,
        )



"""Simple module for handling errors.

    This module contains no functions. It is only used to define the
    Error classes that the other modules use.
"""
from dataclasses import dataclass


@dataclass
class BaseError(BaseException):
    code: int
    msg: str

    def __init__(self, *args: object) -> None:
        super().__init__(f"Error [Code {self.code}]: {self.msg}")


class BadRequest(BaseError):
    code = 400
    msg = "Bad Request"


class Unauthorized(BaseError):
    code = 401
    msg = "Unauthorized"


class Forbidden(BaseError):
    code = 403
    msg = "Forbidden"


class NotFound(BaseError):
    code = 404
    msg = "Not Found"


class Conflict(BaseError):
    code = 409
    msg = "Conflict"


class InternalServerError(BaseError):
    code = 500
    msg = "Internal Server Error"


class NotImplemented(BaseError):
    code = 501
    msg = "Not Implemented"


class BadGateway(BaseError):
    code = 502
    msg = "Bad Gateway"


class ServiceUnavailable(BaseError):
    code = 503
    msg = "Service Unavailable"


class GatewayTimeout(BaseError):
    code = 504
    msg = "Gateway Timeout"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pytenno', 'pytenno._backends', 'pytenno.interface', 'pytenno.models']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.1,<4.0.0']

setup_kwargs = {
    'name': 'pytenno',
    'version': '0.2.2',
    'description': 'A asynchronous Python library for interacting with the warframe.market API',
    'long_description': '# PyTenno\nPyTenno is an asynchronous wrapper for the [warframe.market](https://warframe.market) API, written in Python.\n\n[Read Documentation](https://pytenno.readthedocs.io/en/latest/index.html)\n\n### Disclaimer\nPyTenno is not associated with warframe.market or any of its affiliates.\nPyTenno is not associated with Digital Extremes or any of its affiliates.\n\n## Installation\n\n### Git\nUse your favorite spice of the following:\n\n```bash\npy -m pip install git+https://github.com/ShadowMagic896/pytenno.git\n```\n- [Git-SCM](https://git-scm.com/) is required for direct installation\n\n### PyPi\n\n```bash\npy -m pip install pytenno\n```\n\n## Requirements\nThe project\'s only requirement is aiohttp, which is available on PyPi.\n\n[aiohttp](https://aiohttp.readthedocs.io/en/stable/index.html) >= 3.8.1\n\n## Skill-Set Requirements\n\n### PyTenno Requires Fundamental understanding of the following:\n1. Python datatypes, attributes, etc.\n2. Asyncrhonous programming in Python (Coroutines, `async`, `await`)\n3. Fundamental understanding of context managers in Python (`async with`)\n4. How [warframe.market](https://warframe.market) works \n5. [Warframe](https://warframe.com/) mechanics\n\n## Examples\n\n### The Following Code Will Be Used in the Examples\n```python\n\nimport asyncio # To use asynchronous programming\nimport pytenno\nfrom pytenno.models.enums import Platform # To specify platforms for requests\n\nasync def main(): # PyTenno is asynchronous, so it must be done in an asynchronous context\n    default_language = "en" # Set default response language to English\n    default_platform = Platform.pc # Set default platform to PC\n\n    # Create a client with the default language and platform\n    # This must be done in an asynchronous context manager (async with ... [as ...])\n    async with pytenno.PyTenno(default_language, default_platform) as tenno:\n        ... # Example code goes here\n    \nif __name__ == "__main__":\n    asyncio.run(main()) # Create a new asyncio loop and run the coroutine\n```\n\n### Log into warframe.market\n```python\n        email = "you@example.com" # Email to account\n        password = "password123" # Password to account\n        current_user = await tenno.Auth.login(\n            email=email,\n            password=password\n        ) # Log in to warframe.market\n        # Note: the API stores absoloutely zero information about data passed. All code\n        # is open-source and available at https://github.com/ShadowMagic896/pytenno\n        print(current_user.ingame_name) # Print the ingame name of the user that was logged in\n```\n\n\n',
    'author': 'Ryan Peckham',
    'author_email': '86629697+ShadowMagic896@users.noreply.github.com',
    'maintainer': 'Ryan Peckham',
    'maintainer_email': '86629697+ShadowMagic896@users.noreply.github.com',
    'url': 'https://github.com/ShadowMagic896/pytenno',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
}


setup(**setup_kwargs)



from dataclasses import dataclass

from .enums import Faction


@dataclass
class Location:
    """Represents a location."""

    id: str
    """The ID of the location."""
    icon: str
    """The icon URL of the location."""
    thumb: str
    """The thumbnail URL of the location."""
    faction: Faction
    """The faction of the location."""
    name: str
    """The name of the location."""
    node_name: str
    """The name of the node the location is on."""



"""Module holding the Liches interface class."""

from typing import Optional

from .._backends.liches import LichesBackend
from ..constants import VALID_LANGUAGES
from ..models.liches import LichEphemera, LichQuirk, LichWeapon


class Liches(LichesBackend):
    """Class for the liches backend."""

    async def get_weapons(
        self, language: Optional[VALID_LANGUAGES] = None
    ) -> list[LichWeapon]:
        """Gets all weapons.

        Parameters
        ----------
        language : Optional[VALID_LANGUAGES]
            The language of the weapons. Default: ``None``, meaning the default set during client construction.

        Returns
        -------
        list[LichWeapon]

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     weapons = await pytenno.liches.get_weapons()
        >>>     for weapon in weapons:
        >>>         print(weapon.url_name)
        """
        return await self._get_weapons(language)

    async def get_ephemeras(
        self, language: Optional[VALID_LANGUAGES] = None
    ) -> list[LichEphemera]:
        """Gets all lich ephemeras.

        Parameters
        ----------
        language : Optional[VALID_LANGUAGES]
            The language of the ephemeras. Default: ``None``, meaning the default set during client construction.

        Returns
        -------
        list[LichEphemera]

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     ephemeras = await pytenno.liches.get_ephemeras()
        >>>     for ephemera in ephemeras:
        >>>         print(ephemera.url_name)
        """
        return await self._get_ephemeras(language)

    async def get_quirks(
        self, language: Optional[VALID_LANGUAGES] = None
    ) -> list[LichQuirk]:
        """Gets all lich quirks.

        Parameters
        ----------
        language : Optional[VALID_LANGUAGES]
            The language of the quirks. Default: ``None``, meaning the default set during client construction.

        Returns
        -------
        list[LichQuirk]

        Example
        -------
        >>> async with PyTenno() as pytenno:
        >>>     quirks = await pytenno.liches.get_quirks()
        >>>     for quirk in quirks:
        >>>         print(quirk.url_name)
        """
        return await self._get_quirks(language)



from dataclasses import dataclass
from datetime import datetime


@dataclass
class Bid:
    """Represents a bid on an auction."""

    id: str
    """The ID of the bid."""
    auction: str
    """The ID of the auction the bid is on."""
    user: str
    """The ID of the user who made the bid."""
    value: int
    """The value (platinum) of the bid."""
    created: datetime
    """The date and time the bid was created."""
    updated: datetime
    """The date and time the bid was updated."""



"""Module containg models for items."""

from dataclasses import dataclass

from pytenno.constants import VALID_TRANSLATIONS_RAW
from pytenno.utils import from_data

from .droptable import Drop
from .enums import IconFormat, ItemRarity, Subtype


@dataclass
class LangInItem:
    """Represents an item's localized data."""

    item_name: str
    """The translated name of the item."""
    description: str
    """The translated description of the item."""
    wiki_link: str | None
    """The link to the wiki page of the item."""
    drop: list[Drop]
    """Where the item can be found."""

    def from_data(data: dict):
        data["drop"] = [Drop(**drop) for drop in data["drop"]]
        return LangInItem(**data)


@dataclass(kw_only=True)
class ItemCommon:
    """Common base class that an item can inherit from."""

    id: str
    """The ID of the item."""
    url_name: str
    """The URL name of the item."""
    icon: str
    """The URL of the item's icon."""
    icon_format: IconFormat | None = None
    """The format of the item's icon."""
    sub_icon: str = None
    """The URL of the item's sub icon. For example if the item is part of a set, `icon` will the icon of the set, while `sub_icon` will be the icon of the item in the set."""
    thumb: str
    """The URL of the item's thumbnail."""
    tags: list[str]
    """The tags of the item."""
    mod_max_rank: int | None = None
    """The maximum rank of the item."""
    subtypes: list[Subtype] | None = None
    """The subtypes of the item."""
    cyan_stars: int | None = None
    """The number of cyan stars the item has."""
    amber_stars: int | None = None
    """The number of amber stars the item has."""
    ducats: int | None = None
    """The ducat worth of the item."""

    def __repr__(self):
        return f"<ItemCommon id={self.id} url_name={self.url_name} tags={self.tags}>"


@dataclass(kw_only=True)
class TranslatedItemName:
    """Represents an item's localized name."""

    item_name: str
    """The translated name of the item."""


@dataclass(kw_only=True)
class ItemInOrder(ItemCommon):
    """Represents an item in an order."""

    en: TranslatedItemName
    """The English name of the item."""
    ru: TranslatedItemName
    """The Russian name of the item."""
    ko: TranslatedItemName
    """The Korean name of the item."""
    fr: TranslatedItemName
    """The French name of the item."""
    de: TranslatedItemName
    """The German name of the item."""
    sv: TranslatedItemName
    """The Swedish name of the item."""
    zh_hant: TranslatedItemName
    """The Chinese (Traditional) name of the item."""
    zh_hans: TranslatedItemName
    """The Chinese (Simplified) name of the item."""
    pt: TranslatedItemName
    """The Portuguese name of the item."""
    es: TranslatedItemName
    """The Spanish name of the item."""
    pl: TranslatedItemName
    """The Polish name of the item."""

    def __repr__(self):
        return f"<ItemInOrder id={self.id} url_name={self.url_name} tags={self.tags}>"


@dataclass(kw_only=True)
class ItemFull(ItemInOrder):
    """same as ItemInOrder, but lang related fields contain more info, as well as ``rarity``, ``set_root``, ``mastery_level``, and ``trading_tax`` attributes."""

    set_root: bool
    """Whether the item is part of a set."""
    mastery_level: int
    """The mastery level of the item."""
    rarity: ItemRarity | None = None
    """The rarity of the item. If None, the item does not have any specific rarity."""
    trading_tax: int
    """The trading tax of the item."""
    quantity_for_set: int = None
    """The quantity of the item required to obtain the set."""

    en: LangInItem
    """The English translation of the item."""
    ru: LangInItem
    """The Russian translation of the item."""
    ko: LangInItem
    """The Korean translation of the item."""
    fr: LangInItem
    """The French translation of the item."""
    de: LangInItem
    """The German translation of the item."""
    sv: LangInItem
    """The Swedish translation of the item."""
    zh_hant: LangInItem
    """The Chinese (Traditional) translation of the item."""
    zh_hans: LangInItem
    """The Chinese (Simplified) translation of the item."""
    pt: LangInItem
    """The Portuguese translation of the item."""
    es: LangInItem
    """The Spanish translation of the item."""
    pl: LangInItem
    """The Polish translation of the item."""

    def from_data(data: dict):
        for lang in (
            "en",
            "ru",
            "ko",
            "fr",
            "de",
            "sv",
            "zh_hant",
            "zh_hans",
            "pt",
            "es",
            "pl",
        ):
            data[lang] = LangInItem.from_data(data[lang])
        return from_data(ItemFull, data, False)

    def __repr__(self):
        return f"<ItemFull id={self.id} url_name={self.url_name} tags={self.tags} rarity={self.rarity}>"


@dataclass
class ItemShort:
    """Represents a simplified version of an item."""

    id: str
    """The ID of the item."""
    url_name: str
    """The URL name of the item."""
    thumb: str
    """The URL of the item's thumbnail."""
    item_name: str
    """The name of the item."""
    vaulted: bool | None = None
    """Whether the item is vaulted. Default: `None`, meaning the item cannot be vaulted/unvaulted."""

    def __repr__(self):
        return f"<ItemShort id={self.id} url_name={self.url_name}>"
